/** 									// -o-
 * overlay_demodebug1.pde
 *
 *   A telling taxonomy of Triggers.
 *
 *   void setupOsc()
 *
 *   void defineTriggers()
 *   void setupTriggers()
 *   void displayTriggers()
 *   void dumpTriggers()
 *   void scanTriggers()
 *
 * To establish a trigger:
 *   1  declare 
 *   2  define in defineTriggers(); 
 *        add dynamic system values in setupTriggers() 
 *   3  list in displayTriggers() and dumpTriggers()
 *   4  list in scanTriggers(); add state logic here.
 *
 * ----------------------------------------------------------------------
 * Copyright © 2011, David Reeder.  All rights reserved.
 * Please see the LICENSE file in this distribution for license details.
 * ----------------------------------------------------------------------
 */


// -------------------------------------------- -o--
// setupOsc()
//
// establish port for receiving OSC messages; establish IP address
// and port for sending OSC messages.
//
// CALLED BY: setup()
//
void setupOsc()
{
  // receiving port
  //
  osc = new OscP5(this, 42002); 

  // target IP and port
  //
  osc_target = new NetAddress("127.0.0.1", 57120); 	// sc canonical
}



// -------------------------------------------- -o--
// declare and define triggers
//

Rectangle r_topright, r_bottomright;

Rectangle r_radiate1, r_radiate2, r_radiate3, 
	    r_radiate4, r_radiate5, r_radiate6;

Circle c1, c2, c3, c4, c5, c6;

Slider slid1, slid2;


int enable_rectangles = 1,
    enable_radiate = 1,
    enable_circles = 1,
    enable_sliders = 1;



// CALLED BY: setup()
//
void defineTriggers() 							// -o--
{
  // -o-
  // sliders
  //
  slid1 = new Slider( "/thing1", "red", 
 			400+100, 40, 280, 15, Slider.AXIS_HORIZONTAL );
 			//320+21, 30, 280, 15, Slider.AXIS_HORIZONTAL );
    slid1.valueRange(200,6000).value(1000);
    slid1.enableText(true);
    slid1.knobLag(3000);
    slid1.setDelayMode(2000);

    slid1.triggerAction( new MirrorCycles(slid1) );


  slid2 = new Slider( "/thing1", "turquoise", 
  			400+300, 240, 240, 100, Slider.AXIS_VERTICAL );
  			//320+251, 240, 240, 100, Slider.AXIS_VERTICAL );
    slid2.valueRange(300, 900).value(500);
    slid2.enableText(true);
    slid2.setDelayMode(100);



  // -o-
  // rectangles
  //
  							// top right
  r_topright = new Rectangle("/thing2", "purple", width-51, 0, 50); 		
    r_topright.setDelayMode(250);
    r_topright.enableText(true);

    r_topright.oscMsg("440");
    r_topright.oscBundle();

    r_topright.oscMsg(660);
    r_topright.oscBundle();

    r_topright.oscPath("/thing3");
    r_topright.oscMsg(880);
    r_topright.oscBundle();
  							// bottom right
  r_bottomright = 
  	new Rectangle("/thing2", "limegreen", width-51, height-51, 50); 	

    r_bottomright.setDelayMode(2000);
    r_bottomright.oscPath("/thing3");



  // -o-
  // radiator
  //
				// radiator right side, top to bottom
				// OSC arguments represent guitar tuning: 
				//   [ 164.8, 220, 261.6, 293.6, 392, 659.2 ]
  GeometricIncrement radY = new GeometricIncrement(20, 40);
  int radX = 400,
      rad_delay = 250;
    
  r_radiate1 = new Rectangle("/thing2", "darkblue", radX, radY.next(), 250, 10);
  r_radiate1.oscMsg(659.2); 	// E
  r_radiate1.setDelayMode(rad_delay);

  r_radiate2 = new Rectangle("/thing2", "darkblue", radX, radY.next(), 250, 10);
  r_radiate2.oscMsg(392, 1.2, 3, 4.5); 	// B
  r_radiate2.setDelayMode(rad_delay);

  r_radiate3 = new Rectangle("/thing2", "darkblue", radX, radY.next(), 250, 10);
  r_radiate3.oscMsg(293.6); 	// G
  r_radiate3.setDelayMode(rad_delay);

  r_radiate4 = new Rectangle("/thing2", "darkblue", radX, radY.next(), 250, 10);
  r_radiate4.oscMsg(261.6); 	// D
  r_radiate4.setDelayMode(rad_delay);

  r_radiate5 = new Rectangle("/thing2", "darkblue", radX, radY.next(), 250, 10);
  r_radiate5.oscMsg(220);	// A
  r_radiate5.setDelayMode(rad_delay);

  r_radiate6 = new Rectangle("/thing2", "darkblue", radX, radY.next(), 250, 10);
  r_radiate6.oscMsg(164.8); 	// E
  r_radiate6.setDelayMode(rad_delay);



  // -o-
  // circles
  //
 						// overlapping circles
  ArcPointRadius arc = new ArcPointRadius( 
  				50, 300, 		// x, y
			       	15, 10, 		// radius + inc
			       	40, 0, PI/3, 75); 	// run/rise, &c...
  int 	    	circ_delay = 300;
  FreqHalfInc  	fh = new FreqHalfInc(200);


  c1 = new Circle( "/thing2", "brightlime", 
  		arc.nextX(), arc.nextY(), arc.nextR() );
  c1.oscMsg(fh.next(), 4.1, 5.2, 6.3);
  c1.setDelayMode(circ_delay);
  c1.enableText(true);

  c2 = new Circle( "/thing2", "brightlime", 
		     arc.nextX(), arc.nextY(), arc.nextR() );
  c2.oscMsg(fh.next(), 1, 2, 3);
  c2.setDelayMode(circ_delay);

  c3 = new Circle( "/thing2", "brightlime", 
		     arc.nextX(), arc.nextY(), arc.nextR() );
  c3.oscMsg(fh.next());
  c3.setDelayMode(circ_delay);

  c4 = new Circle( "/thing2", "brightlime", 
		     arc.nextX(), arc.nextY(), arc.nextR() );
  c4.oscMsg(fh.next());
  c4.setDelayMode(circ_delay);

  c5 = new Circle( "/thing2", "brightlime", 
		     arc.nextX(), arc.nextY(), arc.nextR() );
  c5.oscMsg(fh.next());
  //c5.setDelayMode(circ_delay);
  //c5.setDelayMode(Trigger.DELAY_HOLD);
  //c5.oscTrigger(false);

  c6 = new Circle( "/thing2", "brightlime", 
		     arc.nextX(), arc.nextY(), arc.nextR() );
  c6.oscMsg(fh.next());
  //c6.setDelayMode(circ_delay);
  //c6.setDelayMode(Trigger.DELAY_HOLD);
  c6.enableText(true);
  c6.oscTrigger(false);

}  // defineTriggers()




// -------------------------------------------- -o--
// setup, display and dump triggers
//

// CALLED BY: setup()
//
void setupTriggers() 
{
  // define list of colors
  //
  colorlist = new HashMap();

  colorlist.put( "blue", 	#0000ff ); 	// *
  colorlist.put( "greygreen", 	#004411 );
  colorlist.put( "green", 	#00ff00 ); 	// *
  colorlist.put( "turquoise", 	#00ffff ); 	// *
  colorlist.put( "forestgreen", #0f5f3a );
  colorlist.put( "brightlime", 	#0fff55 ); 	// *
  colorlist.put( "skyblue", 	#1080ff ); 	// *
  colorlist.put( "darkblue", 	#111133 );
  colorlist.put( "brown", 	#222200 );
  colorlist.put( "limegreen", 	#4e9a1e ); 	// *
  colorlist.put( "mutedred", 	#771111 );
  colorlist.put( "purple", 	#8400b4 ); 	// *
  colorlist.put( "red", 	#ff0000 ); 	// *

  //colorlist.put( "newcolor", 	#TBD );

}  // setupTriggers()



// XXX  list all triggers intended to appear in the display
//
// CALLED BY: draw()
//
void displayTriggers() 							// -o--
{
  if (1 == enable_rectangles) {
    r_topright.display();
    r_bottomright.display();
  }

  if (1 == enable_sliders) {
    slid1.writing( round100(slid1.value()) );
    slid1.display();

    slid2.writing( round100(slid2.value()) );
    slid2.display();
  }

  if (1 == enable_radiate) {
    r_radiate1.display();
    r_radiate2.display();
    r_radiate3.display();
    r_radiate4.display();
    r_radiate5.display();
    r_radiate6.display();
  }

  if (1 == enable_circles) {
    c1.display();
    c2.display();
    c3.display();
    c4.display();
    //c5.display();
    c6.display();
  }

}  // displayTriggers()



// XXX  list all triggers that should be tracked
//
void dumpTriggers() 							// -o--
{
  println( "\nTRIGGERS--");
  println( "  (runtime: " + timestamp_ms(-1) +")\n" );

  if (1 == enable_rectangles) {
    println(r_topright.toString("r_topright") +"\n");
    println(r_bottomright.toString("r_bottomright") +"\n");
  }

  if (1 == enable_sliders) {
    println(slid1.toString("slid1") +"\n");
    println(slid2.toString("slid2") +"\n");
  }
  
  if (1 == enable_radiate) {
    println(r_radiate1.toString("r_radiate1") +"\n");
    println(r_radiate2.toString("r_radiate2") +"\n");
    println(r_radiate3.toString("r_radiate3") +"\n");
    println(r_radiate4.toString("r_radiate4") +"\n");
    println(r_radiate5.toString("r_radiate5") +"\n");
    println(r_radiate6.toString("r_radiate6") +"\n");
  }

  if (1 == enable_circles) {
    println(c1.toString("c1") +"\n");
    println(c2.toString("c2") +"\n");
    println(c3.toString("c3") +"\n");
    println(c4.toString("c4") +"\n");
    //println(c5.toString("c5") +"\n");
    println(c6.toString("c6") +"\n");
  }

}  // dumpTriggers()




// -------------------------------------------- -o--
// scan triggers and handle algebra of triggers
//
// CALLED BY: draw()
//
boolean  onceonly_rectangles = true,
         onceonly_sliders = true;

int 	 ct = 10;  		// corner threshhold...


void scanTriggers()
{
  // XXX  list all triggers intended to be activated by pixel differences;
  // NB  isActive() calls isDelayed() which maintains delayMode state
  //
  if (1 == enable_rectangles) {
    r_topright.isActive();
    r_bottomright.isActive();
  }

  if (1 == enable_sliders) {
    slid1.isActive();
    slid2.isActive();

/*
    if (slid2.isActive() && onceonly_sliders) 
    {
      onceonly_sliders = false;

      //slid1.valueRange(1000, 2000);
      //slid1.knobLag(1000);

      slid1.triggerAction(new SliderAction(slid1));
      slid1.oscClearBundle();
      slid1.oscMsg(-1); 	// XXX  placeholder
    }
*/
  }

  if (1 == enable_radiate) {
    r_radiate1.isActive();
    r_radiate2.isActive();
    r_radiate3.isActive();
    r_radiate4.isActive();
    r_radiate5.isActive();
    r_radiate6.isActive();
  }

  if (1 == enable_circles) {
    c1.isActive();
    c2.isActive();
    c3.isActive();
    c4.isActive();
    //c5.isActive();
    c6.isActive();
  } 


  // inter-trigger logic
  //
  if (1 == enable_rectangles) 
  {
    r_topright.writing( ct - r_topright.triggerCount() );

    if ((ct/2) > r_topright.triggerCount()) { 
      r_bottomright.oscClearMsg();
      r_bottomright.oscMsg( round100(random(300,500)) );
    
    /*
    } else if (ct > r_topright.triggerCount()) {
      if (1 == enable_sliders) {
	slid1.valueRange(200,600);
	slid2.valueRange(300,900);
      }
    */

    } else { 
      if (onceonly_rectangles) { 
	onceonly_rectangles = false; 

	r_topright.enableText(false);

	r_bottomright.oscClearMsg();
	r_bottomright.oscPath(); 		// revert to oscPath
	r_bottomright.oscMsg(330);

	r_topright.oscClearBundle();
	r_topright.oscMsg(440);

	if (1 == enable_sliders) {
	  slid1.knobDimentions(40,20);
	  slid2.knobDimentions(20,10);
        }
      }
    }
  }

  if (1 == enable_circles) {
    c1.writing( Integer.toString(c1.triggerCount()) );
    c6.writing( Integer.toString(r_topright.triggerCount()) );
  }

}  // scanTriggers()

